const std = @import("std");
const UUID = @import("uuid").UUID;

const RefTable = @import("../containers/table.zig").RefTable;
const vec = @import("../math/vec.zig");
const Vec4f = vec.Vec4f;
const Vec4i = vec.Vec4i;
const VkMeshId = @import("vulkan/meshes.zig").Mesh.Id;

// TODO memory
var gpa = std.heap.GeneralPurposeAllocator(.{}){};
const allocator = gpa.allocator();

const MTable = RefTable(UUID, Mesh);
var s_meshes: MTable = undefined;

pub fn init() void {
    s_meshes = MTable.init(allocator);
}

pub fn deinit() void {
    s_meshes.deinit();
}

pub fn get(id: Mesh.Id) ?*Mesh {
    return s_meshes.getPtr(id);
}

pub const Mesh = packed struct {
    const version = 5;
    pub const Id = MTable.Id;

    positions: [*]Vec4f,
    normals: [*]Vec4f,
    uvs: [*]Vec4f,
    tex_indices: [*]Vec4i,
    length: i32,
    vk_id: VkMeshId,
};

pub const DiskMesh = packed struct {
    version: i32,
    length: i32,
    name: [64]u8,

    pub const Id = packed struct {
        id: UUID,
    };
};
