const std = @import("std");
const vk = @import("vulkan");
const settings = @import("settings.zig");
const Instance = @import("instance.zig").Instance;

const Self = @This();

messenger: if (settings.messenger_on) vk.DebugUtilsMessengerEXT else void,

pub fn init(instance: *Instance) !Self {
    if (settings.messenger_on) {
        const messenger = try instance.dispatch.createDebugUtilsMessengerEXT(instance.handle, &.{
            .flags = .{},
            .message_severity = .{
                .error_bit_ext = true,
                .warning_bit_ext = true,
                .info_bit_ext = true,
            },
            .message_type = .{
                .general_bit_ext = true,
                .validation_bit_ext = true,
                .performance_bit_ext = true,
            },
            .pfn_user_callback = onVulkanMessage,
            .p_user_data = null,
        }, null);
        return Self{ .messenger = messenger };
    } else {
        return Self{};
    }
}

pub fn deinit(self: *const Self, instance: *const Instance) void {
    if (settings.messenger_on) {
        instance.dispatch.destroyDebugUtilsMessengerEXT(instance.handle, self.messenger, null);
    }
}

fn onVulkanMessage(message_severity: u32, message_type: u32, callback_data: ?*const vk.DebugUtilsMessengerCallbackDataEXT, user_data: ?*anyopaque) callconv(.C) u32 {
    //var shouldLog = false;
    // TODO: log severity
    _ = message_severity;
    _ = message_type;
    _ = user_data;
    if (callback_data) |data| {
        std.debug.print("vkr: {s}\n", .{data.p_message});
    }

    return 0;
}
