const vk = @import("vulkan");

// Compile-time settings for Vulkan.

pub const app_name = "efemra";
pub const engine_name = "efemra";

pub const khronos_layer_on = true;
pub const assist_layer_on = true;
pub const messenger_on = true;
pub const rt_on = false;
pub const hdr_on = true;

// number of swapchain images
pub const max_swapchain_len = 3;
pub const desired_swapchain_len = 2;

// number of resources for dynamic buffers and images
pub const resource_sets = 2;
pub const cmds_per_queue = 64;

pub fn OnlyIf(comptime setting: bool, t: anytype) type {
    return if (setting) t else void;
}

pub const Bid = struct {
    id: u32,
    desc_type: vk.DescriptorType,
    count: u32 = 1,
    stages: vk.ShaderStageFlags,

    pub const TextureTable = struct {
        pub const T1D = Bid{
            .id = 0,
            .desc_type = .combined_image_sampler,
            .count = 64,
            .stages = .{ .fragment_bit = true, .compute_bit = true },
        };
        pub const T2D = Bid{
            .id = T1D.id + 1,
            .desc_type = .combined_image_sampler,
            .count = 512,
            .stages = .{ .fragment_bit = true, .compute_bit = true },
        };
        pub const T3D = Bid{
            .id = T2D.id + 1,
            .desc_type = .combined_image_sampler,
            .count = 64,
            .stages = .{ .fragment_bit = true, .compute_bit = true },
        };
        pub const TCube = Bid{
            .id = T3D.id + 1,
            .desc_type = .combined_image_sampler,
            .count = 64,
            .stages = .{ .fragment_bit = true, .compute_bit = true },
        };
        pub const T2DArray = Bid{
            .id = TCube.id + 1,
            .desc_type = .combined_image_sampler,
            .count = 64,
            .stages = .{ .fragment_bit = true, .compute_bit = true },
        };
    };

    pub const Globals = Bid{
        .id = TextureTable.T2DArray.id + 1,
        .desc_type = .uniform_buffer,
        .stages = .{
            .vertex_bit = true,
            .tessellation_control_bit = true,
            .tessellation_evaluation_bit = true,
            .geometry_bit = true,
            .fragment_bit = true,
            .compute_bit = true,
            .task_bit_nv = true,
            .mesh_bit_nv = true,
            .raygen_bit_khr = true,
            .any_hit_bit_khr = true,
            .closest_hit_bit_khr = true,
            .miss_bit_khr = true,
            .intersection_bit_khr = true,
            .callable_bit_khr = true,
            .subpass_shading_bit_huawei = true,
        },
    };
    pub const SceneLuminance = Bid{
        .id = Globals.id + 1,
        .desc_type = .combined_image_sampler,
        .stages = .{ .fragment_bit = true, .compute_bit = true },
    };
    pub const RWSceneLuminance = Bid{
        .id = SceneLuminance.id + 1,
        .desc_type = .storage_image,
        .stages = .{ .compute_bit = true },
    };
    pub const HistogramBuffer = Bid{
        .id = RWSceneLuminance.id + 1,
        .desc_type = .storage_image,
        .stages = .{ .compute_bit = true },
    };
    pub const ExposureBuffer = Bid{
        .id = HistogramBuffer.id + 1,
        .desc_type = .storage_buffer,
        .stages = .{ .fragment_bit = true, .compute_bit = true },
    };

    pub const Count = ExposureBuffer.id + 1;
};

// Bid.Globals
pub const uniform_buffer_count = 1;
// Bid.RWSceneLuminance
pub const storage_image_count = 1;
// Bid.HistogramBuffer
// Bid.ExposureBuffer
pub const storage_buffer_count = 2;

pub const combined_image_sampler_count = Bid.TextureTable.T1D.count +
    Bid.TextureTable.T2D.count +
    Bid.TextureTable.T3D.count +
    Bid.TextureTable.TCube.count +
    Bid.TextureTable.T2DArray.count +
    Bid.SceneLuminance.count; // wasn't .count, probably a bug in pim?
